// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
  "cloudera/form/SimpleTypeahead",
  "underscore",
  "cloudera/Util",
  "knockout"
], function(SimpleTypeahead, _, Util, ko) {

  var BASIC_SEARCH = "basic";
  var ADVANCED_SEARCH = "advanced";

  /**
   * Manages the charts search widget.
   *
   * options = {
   *   container:    (required) "selector or element of the container object",
   *   recentUri:    (required) "the URI to fetch the list of recent queries",
   *   searchUri:    (required) "the URI to construct the current page but with a different tsquery",
   *   metric:       (optional) "the default metric input",
   *   tsquery:      (optional) "the default tsquery value",
   *   context:      (optional) the context object in edit/add mode
   * };
   */
  function ChartsSearch(options) {

    var self = this, $container = $(options.container);

    // This URL is subject to change.
    var typeaheadUri = "/cmf/charts/typeahead";

    var $metricInput = $container.find(".metricInput");
    var $tsqueryInput = $container.find(".tsqueryInput");

    self.tsquery = ko.observable(options.tsquery || "");

    if (_.isEmpty(options.tsquery)) {
      self.searchMode = ko.observable(BASIC_SEARCH);
      $metricInput.focus();
    } else {
      self.searchMode = ko.observable(ADVANCED_SEARCH);
      $tsqueryInput.focus();
    }

    $tsqueryInput.keypress(function(evt) {
      // if user pressed enter, ignore it,
      // we need this to prevent new line characters
      // from appearing in the query.
      if ($.ui.keyCode.ENTER === evt.which) {
        evt.preventDefault();
      }
    });

    $tsqueryInput.keyup(function(evt) {
      // if user pressed enter, triggers search
      // automatically.
      if ($.ui.keyCode.ENTER === evt.which) {
        self.handleClick();
        evt.preventDefault();
      }
    });


    $metricInput.SimpleTypeahead({
      url: typeaheadUri + "?limit=8",
      value: 'label',
      name: 'value',
      updater: function(name, value) {
        self.clearInputError($metricInput);
        self.tsquery(name.tsquery);
        // When user selects a metric, we perform
        // search automatically.
        self.handleClick();
        return value;
      },
      noResults: function() {
        // If there were no results in the basic query box, clear the advanced
        // box so we don't submit invalid results.
        $tsqueryInput.val('');
        self.tsquery('');
      }
    });

    $.subscribe('chartSearch.addMetric', function(metricName) {
      var query = 'select ' + metricName;
      $metricInput.val('');
      $tsqueryInput.val(query);
      self.tsquery(query);
      self.searchMode(ADVANCED_SEARCH);
      self.handleClick();
    });

    self.selectBasicMode = function() {
      self.searchMode(BASIC_SEARCH);
    };

    self.selectAdvancedMode = function() {
      self.searchMode(ADVANCED_SEARCH);
    };

    self.alertInputError = function($input) {
      $input.closest(".control-group").addClass("error");
    };

    self.clearInputError = function($input) {
      $input.closest(".control-group").removeClass("error");
    };

    self.contextKeyValues = ko.observableArray();
    _.each(options.context, function(v, k) {
      self.contextKeyValues.push({
        name: k,
        value: v
      });
    });

    /**
     * Validates the form and returns true if
     * the form is valid.
     */
    self.validate = function() {
      if (self.searchMode() === ADVANCED_SEARCH) {
        return self.validateAdvanced();
      } else {
        return self.validateBasic();
      }
    };

    /**
     * Validates the basic form.
     */
    self.validateBasic = function() {
      var showError = function() {
        $metricInput.focus();
        self.alertInputError($metricInput);
        return false;
      };
      if ($metricInput.val() === "") {
        return showError();
      } else if (self.tsquery() === "") {
        // If the tsquery is blank, the user entered an invalid basic query.
        return showError();
      } else {
        self.clearInputError($metricInput);
        return true;
      }
    };

    /**
     * Validates the advanced form.
     */
    self.validateAdvanced = function() {
      self.tsquery($tsqueryInput.val());

      if (self.tsquery() === "") {
        $tsqueryInput.focus();
        self.alertInputError($tsqueryInput);
        return false;
      } else {
        self.clearInputError($tsqueryInput);
        return true;
      }
    };

    function RecentQuery(tsquery) {
      this.name = tsquery;
      this.href = options.searchUri + "?tsquery=" + encodeURIComponent(tsquery);
    }

    /**
     * Handles the execution click.
     * Fires the "tsqueryChanged" event.
     */
    self.handleClick = function() {
      if (self.validate()) {
        var existing = _.filter(self.recentList(), function(recentQuery) {
          return recentQuery.name === self.tsquery();
        });
        if (existing.length === 0) {
          self.recentList.unshift(new RecentQuery(self.tsquery()));
        }
        $.publish("tsqueryChanged", [self.tsquery()]);
      }
    };

    self.recentList = ko.observableArray();

    self.getRecentList = function() {
      $.get(options.recentUri, function(response) {
        var filteredJsonResponse = Util.filterJsonResponseError(response);
        _.each(filteredJsonResponse, function(tsquery) {
          self.recentList.push(new RecentQuery(tsquery));
        });
      });
    };

    self.getRecentList();

    ko.applyBindings(self, $(options.container)[0]);
  }

  return ChartsSearch;
});
